<?PHP
/* functions for outputting to html and javascript */

require('chess.inc');

############################################################
function drawboard(){
############################################################
    global $board, $playersColor, $numMoves, $history, $isGameOver;
    global $CFG_BOARDSQUARESIZE;

    /* old PHP versions don't have _POST, _GET and _SESSION as auto_globals */
    if (!minimum_version("4.1.0"))
    	global $_POST, $_GET, $_SESSION;

    /* find out if it's the current player's turn */
    if (( (($numMoves == -1) || (($numMoves % 2) == 1))
		&& ($playersColor == "white")) || ((($numMoves % 2) == 0)
		&& ($playersColor == "black")) )
	$isPlayersTurn = true;
    else
	$isPlayersTurn = false;

    /* determine who's perspective of the board to show */
    if ($_SESSION['isSharedPC'] && !$isPlayersTurn){
	if ($playersColor == "white")
	    $perspective = "black";
	else
	    $perspective = "white";
    }else{
	$perspective = $playersColor;
    }
		
    /* NOTE: if both players are using the same PC, in a sense it's always the players turn */
    if ($_SESSION['isSharedPC'])
	$isPlayersTurn = true;
		
    /* determine if board is disabled */
    $isDisabled = isBoardDisabled();

    if ($isDisabled) {
	// display replay controls
	if($isGameOver){
	echo ("<table width='450' border='0'><tr>");
        echo ("<td align='left'><a href='#' onclick='MoveToStart(); return false;'>|&lt; </a></td>");
        echo ("<td align='left'><a href='#' onclick='BackMove(); return false;'>&lt;&lt; </a></td>");
        echo ("<td align='right'><a href='#' onclick='NextMove(); return false;'>&gt;&gt;</a></td>");
        echo ("<td align='right'><a href='#' onclick='MoveToEnd(); return false;'>&gt;|</a></td>");
        echo ("</tr></table>");
	}
	echo ("<table border='1'>\n");
	echo ("<tr class='header_diabled'>");
    }else{
	echo ("<table border='1' >\n");
	echo ("<tr class='header_enabled'>");
    }

    /* setup vars to show player's perspective of the board */
    if ($perspective == "white"){
	$topRow = 7;
	$bottomRow = 0;
	$rowStep = -1;
	$leftCol = 0;
	$rightCol = 7;
	$colStep = 1;
    }else{
	$topRow = 0;
	$bottomRow = 7;
	$rowStep = 1;
	$leftCol = 7;
	$rightCol = 0;
	$colStep = -1;
    }
	
    /* column headers */
    echo ("<th>&nbsp;</th>");
    /* NOTE: end condition is ($rightCol + $colStep) since we want to output $rightCol */
    for ($i = $leftCol; $i != ($rightCol + $colStep); $i += $colStep)
	echo ("<th>".chr($i + 97)."</th>");
    echo ("</tr>\n");

    /* for each row... */
    /* NOTE: end condition is ($bottomRow + $rowStep) since we want to output $bottomRow */
    for ($i = $topRow; $i != ($bottomRow + $rowStep); $i += $rowStep){
	echo ("<tr>\n");
	if ($isDisabled)
	    echo ("<th width='20' class='header_diabled'>".($i+1)."</th>\n");
	else
	    echo ("<th width='20' class='header_enabled'>".($i+1)."</th>\n");

	/* for each col... */
	/* NOTE: end condition is ($rightCol + $colStep) since we want to output $rightCol */
	for ($j = $leftCol; $j != ($rightCol + $colStep); $j += $colStep){
	    echo ("   <td class='");
	    /* if board is disabled, show board in grayscale */
	    if ($isDisabled){
		if (($j + ($i % 2)) % 2 == 0)
		    echo ("light_disabled'>");
		else
		    echo ("dark_disabled'>");
	    }else{
		if (($j + ($i % 2)) % 2 == 0)
		    echo ("light_enabled");
		else
		    echo ("dark_enabled");
						
  		// smejo: highlight the last move (modify also chess.css)
  		if ($numMoves > -1 &&
  		           (($history[$numMoves]['fromCol'] == $j && $history[$numMoves]['fromRow'] == $i) ||
  		           ($history[$numMoves]['toCol'] == $j && $history[$numMoves]['toRow'] == $i)))
  		{
  		    echo("_moved");
  		}
		echo("'>");
	    }

	    /* if disabled or not player's turn, can't click pieces */
	    if (!$isDisabled && $isPlayersTurn)	{
		echo ("<a href='JavaScript:squareClicked($i, $j, ");
		if ($board[$i][$j] == 0)
		    echo ("true)'>");
		else
		    echo ("false)'>");
	    }
                        
            $l = $i + 1;
            $k = chr($j + 97);
	    echo ("<img name='pos$i-$j' id='$k$l' src='images/".$_SESSION['pref_theme']."/");

	    /* if position is empty... */
	    if ($board[$i][$j] == 0){
		/* draw empty square */
		$tmpALT="blank";
	    }else{
		/* draw correct piece */
		if ($board[$i][$j] & BLACK)
		    $tmpALT = "black_";
		else
		    $tmpALT = "white_";

		$tmpALT .= getPieceName($board[$i][$j]);
	    }

	    echo($tmpALT.".gif' height='$CFG_BOARDSQUARESIZE' width='$CFG_BOARDSQUARESIZE' border='0' alt='".$tmpALT."'>");
	    if (!$isDisabled && $isPlayersTurn)
		echo ("</a>");
	    echo ("</td>\n");
	}
	echo ("</tr>\n");
    }
    echo ("</table>\n\n");
}

############################################################
function writeJSboard(){
############################################################
    global $board, $numMoves;
    /* old PHP versions don't have _POST, _GET and _SESSION as auto_globals */
    if (!minimum_version("4.1.0"))
	global $_POST, $_GET, $_SESSION;
		
    /* write out constants */
    echo ("var DEBUG = ".DEBUG.";\n");
    echo ("var CURRENTTHEME = '".$_SESSION['pref_theme']."';\n");
    echo ("var PAWN = ".PAWN.";\n");
    echo ("var KNIGHT = ".KNIGHT.";\n");
    echo ("var BISHOP = ".BISHOP.";\n");
    echo ("var ROOK = ".ROOK.";\n");
    echo ("var QUEEN = ".QUEEN.";\n");
    echo ("var KING = ".KING.";\n");
    echo ("var BLACK = ".BLACK.";\n");
    echo ("var WHITE = ".WHITE.";\n");
    echo ("var COLOR_MASK = ".COLOR_MASK.";\n");
		
    /* write code for array */
    echo ("var board = new Array();\n");
    for ($i = 0; $i < 8; $i++){
	echo ("board[$i] = new Array();\n");
	for ($j = 0; $j < 8; $j++){
	    echo ("board[$i][$j] = ".$board[$i][$j].";\n");
	}
    }
    echo("var numMoves = $numMoves;\n");
    echo("var errMsg = '';\n");	/* global var used for error messages */
}


/* provide history data to javascript function */
/* NOTE: currently, only pawn validation script uses history */
############################################################
function writeJSHistory(){
############################################################
    global $history, $numMoves;

    /* write out constants */
    echo ("var CURPIECE = 0;\n");
    echo ("var CURCOLOR = 1;\n");
    echo ("var FROMROW = 2;\n");
    echo ("var FROMCOL = 3;\n");
    echo ("var TOROW = 4;\n");
    echo ("var TOCOL = 5;\n");
		
    /* write code for array */
    echo ("var chessHistory = new Array();\n");
    for ($i = 0; $i <= $numMoves; $i++){
	echo ("chessHistory[$i] = new Array();\n");
	echo ("chessHistory[$i][CURPIECE] = '".$history[$i]['curPiece']."';\n");
	echo ("chessHistory[$i][CURCOLOR] = '".$history[$i]['curColor']."';\n");
	echo ("chessHistory[$i][FROMROW] = ".$history[$i]['fromRow'].";\n");
	echo ("chessHistory[$i][FROMCOL] = ".$history[$i]['fromCol'].";\n");
	echo ("chessHistory[$i][TOROW] = ".$history[$i]['toRow'].";\n");
	echo ("chessHistory[$i][TOCOL] = ".$history[$i]['toCol'].";\n");
    }
}
	
############################################################
function writeVerbousHistory(){
############################################################
    global $history, $numMoves;

    echo ("<table width='220' border='1'>\n");
    echo ("<tr><th class='move_header' colspan='2'>HISTORY (".$_SESSION['gameID'].")</th></tr>\n");
    for ($i = $numMoves; $i >= 0; $i--){
	if ($i % 2 == 1){
	    echo ("<tr class='move_black'>");
	    echo ("<td width='20'>".($i + 1)."</td><td>");
	}else{
	    echo ("<tr class='move_white'>");
	    echo ("<td width='20'>".($i + 1)."</td><td>");
	}
	$tmpReplaced = "";
	if (!is_null($history[$i]['replaced']))
	    $tmpReplaced = $history[$i]['replaced'];
			
	$tmpPromotedTo = "";
	if (!is_null($history[$i]['promotedTo']))
	    $tmpPromotedTo = $history[$i]['promotedTo'];

	$tmpCheck = ($history[$i]['isInCheck'] == 1);
			
	echo(moveToVerbousString($history[$i]['curColor'], $history[$i]['curPiece'], $history[$i]['fromRow'], $history[$i]['fromCol'], $history[$i]['toRow'], $history[$i]['toCol'], $tmpReplaced, $tmpPromotedTo, $tmpCheck));
			
	echo ("</td></tr>\n");
    }
    echo ("<tr class='move_header'><td>0</td><td>New Game</td></tr>\n");
    echo ("</table>\n");
}

############################################################
function writeHistoryPGN(){
############################################################
    global $history, $numMoves;
    echo ("<table border='1'>\n");
    echo ("<tr><th class='move_header' colspan='3'>HISTORY (".$_SESSION['gameID'].")</th></tr>\n");
    echo ("<tr><th class='move_move' width='50'>Move</th>");
    echo ("<th class='move_white' width='80'>White</th>");
    echo ("<th class='move_black' width='80'>Black</th></tr>\n");

    for ($i = 0; $i <= $numMoves; $i+=2){
	echo ("<tr><td class='move_move'>");
	if (isBoardDisabled())
	    echo ("<a href='#' onclick='MoveToPosition(".$i."); return false;'>".(($i/2) + 1)."</a>");
	else
	    echo (($i/2) + 1);
	echo ("</td><td class='move_white'>");
	echo ("<div id='move_".$i."'>");

	$tmpReplaced = "";
	if (!is_null($history[$i]['replaced']))
	    $tmpReplaced = $history[$i]['replaced'];
			
	$tmpPromotedTo = "";
	if (!is_null($history[$i]['promotedTo']))
	    $tmpPromotedTo = $history[$i]['promotedTo'];

	$tmpCheck = ($history[$i]['isInCheck'] == 1);
			
	echo(moveToPGNString($history[$i]['curColor'], $history[$i]['curPiece'], $history[$i]['fromRow'], $history[$i]['fromCol'], $history[$i]['toRow'], $history[$i]['toCol'], $tmpReplaced, $tmpPromotedTo, $tmpCheck));

	echo ("</div>");
	echo ("</td><td class='move_black'>");
	echo ("<div id='move_".($i + 1)."'>");
	if ($i == $numMoves)
	    echo ("&nbsp;");
	else{
	    $tmpReplaced = "";
	    if (!is_null($history[$i+1]['replaced']))
		$tmpReplaced = $history[$i+1]['replaced'];
			
	    $tmpPromotedTo = "";
	    if (!is_null($history[$i+1]['promotedTo']))
	    	$tmpPromotedTo = $history[$i+1]['promotedTo'];

	    $tmpCheck = ($history[$i+1]['isInCheck'] == 1);
			
	    echo(moveToPGNString($history[$i+1]['curColor'], $history[$i+1]['curPiece'], $history[$i+1]['fromRow'], $history[$i+1]['fromCol'], $history[$i+1]['toRow'], $history[$i+1]['toCol'], $tmpReplaced, $tmpPromotedTo, $tmpCheck));
	}
	echo ("</div>");
	echo ("</td></tr>\n");
    }
    echo ("</table>\n");
}

############################################################
function writeHistory(){
############################################################
    /* old PHP versions don't have _POST, _GET and _SESSION as auto_globals */
    if (!minimum_version("4.1.0"))
	global $_POST, $_GET, $_SESSION;
		
    /* based on player's preferences, display the history */
    switch($_SESSION['pref_history']){
	case 'verbous':
	    writeVerbousHistory();
	    break;
	case 'pgn':
	    writeHistoryPGN();
	    break;
    }
}

############################################################
function writeStatus(){
############################################################
    global $numMoves, $history, $isCheckMate, $statusMessage, $isPlayersTurn;
    global $isPromoting,$isUndoRequested,$isDrawRequested;
    echo "<form name='gamedata2' method='post' action='chess.php'>\n";
    echo "<table border='1' width='220' align='center'>\n";
    echo " <tr bgcolor='beige'>\n";
    echo "  <th class='stat_header'>\n";
    echo "    STATUS - \n";
    if ($isPlayersTurn) 
	echo "Your Move"; 
    else 
	echo "Opponent's Move"; 
    echo "  </th>\n";
    echo " </tr>\n";

    if (($numMoves == -1) || ($numMoves % 2 == 1))
	$curColor = "White";
    else			
	$curColor = "Black";


    if ($isPromoting){
   	echo "<tr class='tr_stat' ><td>\n";
        writePromotion();
     	echo "</td>\n";
	echo " </tr>\n";
    }
    if ($isUndoRequested){
	echo "<tr class='tr_stat' ><td>\n";
    	writeUndoRequest();
	echo "</td>\n";
	echo " </tr>\n";
    }
    if ($isDrawRequested){
	echo "<tr class='tr_stat' ><td>\n";
    	writeDrawRequest();
	echo "</td>\n";
	echo " </tr>\n";
    }

    if (!$isCheckMate && ($history[$numMoves]['isInCheck'] == 1))
	echo("<tr class='tr_stat' ><td align='center' bgcolor='red'>\n<b>".$curColor." is currently in check!</b><br>\n".$statusMessage."</td></tr>\n");
    else{
	if ($statusMessage!="")
	    echo("<tr class='tr_stat' ><td>".$statusMessage."&nbsp;</td></tr>\n");
    }
    echo "</table>\n";
    echo "</form>\n";
}

############################################################
function writePromotion(){
############################################################
    echo "<table width='220' border='1'>\n";
    echo " <tr><th class='request'>PROMOTE PAWN TO:\n";
    echo " </th></tr>\n";
    echo " <tr class='tr_stat'><td align='center'>\n";
    echo "  <input type='radio' name='promotion' value='".QUEEN."' checked='checked'> Queen<br>\n";
    echo "  <input type='radio' name='promotion' value='".ROOK."'> Rook<br>\n";
    echo "  <input type='radio' name='promotion' value='".KNIGHT."'> Knight<br>\n";
    echo "  <input type='radio' name='promotion' value='".BISHOP."'> Bishop<br>\n";

        echo "    <input type='hidden' name='fromRow' value='";
    if (isPromoting) echo $_POST['fromRow']; 
    echo "'>\n";
    echo "    <input type='hidden' name='fromCol' value='";
    if (isPromoting) echo $_POST['fromCol'];
    echo"'>\n";
    echo "    <input type='hidden' name='toRow' value='";
    if (isPromoting) echo $_POST['toRow']; 
    echo"'>\n";
    echo "    <input type='hidden' name='toCol' value='";
    if (isPromoting) echo $_POST['toCol'];
    echo"'>\n";
    echo "    <input type='hidden' name='isInCheck' value='false'>\n";
    echo "    <input type='hidden' name='isCheckMate' value='false'>\n";

    echo "  <input type='button' name='btnPromote' value='Promote' onClick='promotepawn()'>\n";
    echo " </td></tr>\n";
    echo "</table>\n";
}

############################################################
function writeUndoRequest(){
############################################################
    echo "<table width='220' border='1'>\n";
    echo " <tr><th class='request'>UNDO REQUEST!\n";
    echo " </th></tr>\n";
    echo " <tr class='tr_stat'><td align='center' >\n";
    echo "Will you allow it?<br>\n";
    echo "  <input type='radio' name='undoResponse' value='yes'> Yes\n";
    echo "  <input type='radio' name='undoResponse' value='no' checked='checked'> No<br>\n";
    echo "  <input type='hidden' name='isUndoResponseDone' value='no'>\n";
    echo "  <input type='button' value='Reply' onClick=\"this.form.isUndoResponseDone.value = 'yes'; this.form.submit()\">\n";
    echo " </td></tr>\n";
    echo "</table>\n";
}

############################################################
function writeDrawRequest(){
############################################################
    echo "<table width='220' border='1'>\n";
    echo " <tr><th class='request'>DRAW REQUEST!\n";
    echo " </th></tr>\n";
    echo " <tr class='tr_stat' ><td align='center' >\n";
    echo "Do you agree?<br>\n";
    echo "  <input type='radio' name='drawResponse' value='yes'> Yes\n";
    echo "  <input type='radio' name='drawResponse' value='no' checked='checked'> No<br>\n";
    echo "  <input type='hidden' name='isDrawResponseDone' value='no'>\n";
    echo "  <input type='button' value='Reply' onClick=\"this.form.isDrawResponseDone.value = 'yes'; this.form.submit()\">\n";
    echo " </td></tr>\n";
    echo "</table>\n";
}

############################################################
function writePGN(){
############################################################
    global $_SESSION, $history, $numMoves,$pWhite,$pWhiteF,$pWhiteL,$pBlack,$pBlackF,$pBlackL,$gStart;
    global $simplemove,$isDraw,$move_turn,$move_PlyNumber;
    $FEN="rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1";
    set_FEN($FEN);
    $xheader ="[Event \"WebChess Board #{$_SESSION['gameID']}\"]\n";
    $xheader .="[Site \"WebChess\"]\n";
    $xheader .="[Date \"$gStart\"]\n";
    $xheader .="[Round \"-\"]\n";
    $xheader .="[White \"$pWhiteL, $pWhiteF\"]\n";
    $xheader .="[Black \"$pBlackL, $pBlackF\"]\n";
    $body="";
    for ($i = 0; $i <= $numMoves; $i+=2){
        $body.="".(($i/2)+1).". ";
	$tmpReplaced = "";
	if (!is_null($history[$i]['replaced']))
	    $tmpReplaced = $history[$i]['replaced'];
			
	$tmpPromotedTo = "";
	if (!is_null($history[$i]['promotedTo']))
	    $tmpPromotedTo = $history[$i]['promotedTo'];

	$tmpCheck = ($history[$i]['isInCheck'] == 1);
	$fr=$history[$i]['fromRow'];
	$fc=$history[$i]['fromCol'];
	$tr=$history[$i]['toRow'];
	$tc=$history[$i]['toCol'];
	$fs=coordsToSquare($fr,$fc);
	$ts=coordsToSquare($tr,$tc);
	add_Move("$fs-$ts".getPGNCode($tmpPromotedTo));
	add_Move("$fs-$ts".getPGNCode($tmpPromotedTo));
	$body.=$simplemove;
	$body.=" ";
	if ($i == $numMoves)
	    $body.=" ";
	else{
	    $tmpReplaced = "";
	    if (!is_null($history[$i+1]['replaced']))
		$tmpReplaced = $history[$i+1]['replaced'];
			
	    $tmpPromotedTo = "";
	    if (!is_null($history[$i+1]['promotedTo']))
		$tmpPromotedTo = $history[$i+1]['promotedTo'];

	    $tmpCheck = ($history[$i+1]['isInCheck'] == 1);
	    $fr=$history[$i+1]['fromRow'];
	    $fc=$history[$i+1]['fromCol'];
	    $tr=$history[$i+1]['toRow'];
	    $tc=$history[$i+1]['toCol'];
	    $fs=coordsToSquare($fr,$fc);
	    $ts=coordsToSquare($tr,$tc);
	    add_Move("$fs-$ts".getPGNCode($tmpPromotedTo));
	    $body.=$simplemove;
	}
	$body.="\n";
    }
    $result=getResult();
    if($isDraw){
	$result="1/2-1/2";
	$body.="{Draw Agreed}";
    };
    $xheader .="[Result \"$result\"]\n\n";
    echo ($xheader);
    echo ($body);
}

############################################################
function writeJSReplayHistory(){
############################################################
   global $history, $numMoves;

   /* old PHP versions don't have _POST, _GET and _SESSION as auto_globals */
   if (!minimum_version("4.1.0"))
      global $_POST, $_GET, $_SESSION;
      
   $movesStr = "";
   // this loop is copied from writeHistoryPGN() and lightly modified
   for ($i = 0; $i <= $numMoves; $i+=2){
      $tmpReplaced = "";
      if (!is_null($history[$i]['replaced']))
         $tmpReplaced = $history[$i]['replaced'];
      
      $tmpPromotedTo = "";
      if (!is_null($history[$i]['promotedTo']))
         $tmpPromotedTo = $history[$i]['promotedTo'];
      
      $tmpCheck = ($history[$i]['isInCheck'] == 1);
      $movesStr .= moveToPGNString2($history[$i]['curColor'], $history[$i]['curPiece'], $history[$i]['fromRow'], $history[$i]['fromCol'], $history[$i]['toRow'], $history[$i]['toCol'], $tmpReplaced, $tmpPromotedTo, $tmpCheck);
      $movesStr .= " ";

      if ($i == $numMoves)
         echo (" ");
      else{
         $tmpReplaced = "";
         if (!is_null($history[$i+1]['replaced']))
            $tmpReplaced = $history[$i+1]['replaced'];
         
         $tmpPromotedTo = "";
         if (!is_null($history[$i+1]['promotedTo']))
            $tmpPromotedTo = $history[$i+1]['promotedTo'];
         
         $tmpCheck = ($history[$i+1]['isInCheck'] == 1);
         $movesStr .= moveToPGNString2($history[$i+1]['curColor'], $history[$i+1]['curPiece'], $history[$i+1]['fromRow'], $history[$i+1]['fromCol'], $history[$i+1]['toRow'], $history[$i+1]['toCol'], $tmpReplaced, $tmpPromotedTo, $tmpCheck);
      }
      $movesStr .= " ";
   }
   //echo $movesStr;
   $movesStr = trim($movesStr);
   // the order here is very important
   $notationStr = str_replace( "W", "", $movesStr );  // white castling
   $movesStr = str_replace( "+", "", $movesStr );
   $movesStr = str_replace( "WO-O-O", "e1a1", $movesStr ); // white queen-side castling
   $movesStr = str_replace( "WO-O", "e1h1", $movesStr );   // white king-side castling
   $movesStr = str_replace( "O-O-O", "e8a8", $movesStr );
   $movesStr = str_replace( "O-O", "e8h8", $movesStr );
   $movesStr = str_replace( "x", "", $movesStr );
   $movesStr = str_replace( "=", "", $movesStr );
   //$movesStr = str_replace( "N", "", $movesStr );
   //$movesStr = str_replace( "B", "", $movesStr );
   //$movesStr = str_replace( "R", "", $movesStr );
   //$movesStr = str_replace( "Q", "", $movesStr );
   //$movesStr = str_replace( "K", "", $movesStr );
   //echo $movesStr;
   //$actualTurn = $numMoves + 1;
   $actualTurn = 0;
   echo("<script type=\"text/javascript\">\n");
   echo("// data for replay.js script\n");
   echo("var actual_turn = $actualTurn;\n");
   echo("var figures_img_path = '".$_SESSION['pref_theme']."';\n");
   echo("var moves_array = new Array();\n");
   echo("var notation_array = new Array();\n");
   echo("var moves_archive_array = new Array();\n");
   echo("var moves_empasant_array = new Array();\n");
   echo("var move_not_done = true;\n");
   echo("var ie4 = (document.all)? true:false;\n");
   echo("var moves_string = '$movesStr';\n");
   echo("var notation_string = '$notationStr';\n");
   echo("</script>\n");
   echo("<script type=\"text/javascript\" src=\"javascript/replay.js\">\n");
   echo("// these functions are used to replay history of the game\n");
   echo("</script>\n");
}

?>
